'-----------------------
macro Peak_Split_2Th_displacement(beam_SD_V, beam_SD)
{
' The "Peak_Split_2Th_displacement" macro describes the peak splitting of the Bragg reflections of data collected in scanning mode in a physically meaningful way.
' Authors: Paolo P. Mazzeo, University of Parma - paolopio.mazzeo[at]unipr.it; Giulio I. Lampronti, University of Cambridge - gil21[at]cam.ac.uk
' Please cite Nat Commun 12, 6134 (2021). https://doi.org/10.1038/s41467-021-26264-1

#m_argu beam_SD_V
If_Prm_Eqn_Rpt(beam_SD_V, beam_SD, min = Val-.1; max = Val+.1; del 0.001)
th2_offset = (Rad * ArcSin((CeV(beam_SD_V, beam_SD) * Sin(2 Th))/Rs));
}
'-----------------------
 
'-----------------------
macro Peak_Split_2Th_displacement_2D(beam_SD_V, beam_SD, m_V, m)
{
' The "Peak_Split_2Th_displacement_2D" macro describes the peak splitting of the Bragg reflections of data collected with large flat area detectors in a physically meaningful way, including an extra empirical correction tunable for the detector dimensions. 
' Authors: Paolo P. Mazzeo, University of Parma - paolopio.mazzeo[at]unipr.it; Giulio I. Lampronti, University of Cambridge - gil21[at]cam.ac.uk
' Please cite Nat Commun 12, 6134 (2021). https://doi.org/10.1038/s41467-021-26264-1

#m_argu beam_SD_V
If_Prm_Eqn_Rpt(beam_SD_V, beam_SD, min = Val-.1; max = Val+.1; del 0.001)
#m_argu m_V
If_Prm_Eqn_Rpt(m_V, m, min = -10; max = 10; del 0.001)
th2_offset = (Rad * ArcSin(((CeV(beam_SD_V, beam_SD) * Sin(2 Th))/Rs) + (CeV(m_V, m) * (Tan(2 Th))^2)));
}
'------------------------

'------------------------
macro TCHZ_Split_Peak_Type(u_Left, u_Leftv, v_Leftv, v_Left, w_Left, w_Leftv, z_Left, z_Leftv, x_Left, x_Leftv, y_Left, y_Leftv, u_Right, u_Rightv, v_Right, v_Rightv, w_Right, w_Rightv, z_Right, z_Rightv, x_Right, x_Rightv, y_Right, y_Rightv)
   {
' The "TCHZ_Split_Peak_Type" macro models the asymmetric peak shape by splitting the TCHZ peak type into left and right sides.
' Authors: Paolo P. Mazzeo, University of Parma - paolopio.mazzeo[at]unipr.it; Giulio I. Lampronti, University of Cambridge - gil21[at]cam.ac.uk
' Please cite Nat Commun 12, 6134 (2021). https://doi.org/10.1038/s41467-021-26264-1

      #m_argu u_Left
      #m_argu v_Left
      #m_argu w_Left
      #m_argu z_Left
      #m_argu x_Left
      #m_argu y_Left
      #m_argu u_Right
      #m_argu v_Right
      #m_argu w_Right
      #m_argu z_Right
      #m_argu x_Right
      #m_argu y_Right
      If_Prm_Eqn_Rpt(u_Left, u_Leftv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(v_Left, v_Leftv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(w_Left, w_Leftv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(z_Left, z_Leftv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(x_Left, x_Leftv, min = Max(0.0001, Val-.1); max = Min(2, Val+.1); del 1.0e-4 )
      If_Prm_Eqn_Rpt(y_Left, y_Leftv, min = Max(0.0001, Val-.1); max = Min(2, Val+.1); del 1.0e-4 )
      If_Prm_Eqn_Rpt(u_Right, u_Rightv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(v_Right, v_Rightv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(w_Right, w_Rightv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(z_Right, z_Rightv, min = Max(-1, Val-.1); max = Min(2, Val+.1); del 1.0e-4)
      If_Prm_Eqn_Rpt(x_Right, x_Rightv, min = Max(0.0001, Val-.1); max = Min(2, Val+.1); del 1.0e-4 )
      If_Prm_Eqn_Rpt(y_Right, y_Rightv, min = Max(0.0001, Val-.1); max = Min(2, Val+.1); del 1.0e-4 )
      local #m_unique tch_p_l_Left = CeV(x_Left, x_Leftv) Tan(Th) + CeV(y_Left, y_Leftv) / Cos(Th);
      local #m_unique tch_p_g_Left = Sqrt( Abs( CeV(u_Left, u_Leftv) Tan(Th)^2 + CeV(v_Left, v_Leftv) Tan(Th) + CeV(w_Left, w_Leftv) + CeV(z_Left, z_Leftv) / Cos(Th)^2) );
      local #m_unique tch_p_Left =
         (
                    tch_p_g_Left^5 +
            2.69269 tch_p_g_Left^4 tch_p_l_Left +
            2.42843 tch_p_g_Left^3 tch_p_l_Left^2 +
            4.47163 tch_p_g_Left^2 tch_p_l_Left^3 +
            0.07842 tch_p_g_Left   tch_p_l_Left^4 +
                              tch_p_l_Left^5
         )^0.2;
      local #m_unique tch_q_Left = tch_p_l_Left / tch_p_Left;
      local #m_unique tch_p_l_Right = CeV(x_Right, x_Rightv) Tan(Th) + CeV(y_Right, y_Rightv) / Cos(Th);
      local #m_unique tch_p_g_Right = Sqrt( Abs( CeV(u_Right, u_Rightv) Tan(Th)^2 + CeV(v_Right, v_Rightv) Tan(Th) + CeV(w_Right, w_Rightv) + CeV(z_Right, z_Rightv) / Cos(Th)^2) );
      local #m_unique tch_p_Right =
         (
                    tch_p_g_Right^5 +
            2.69269 tch_p_g_Right^4 tch_p_l_Right +
            2.42843 tch_p_g_Right^3 tch_p_l_Right^2 +
            4.47163 tch_p_g_Right^2 tch_p_l_Right^3 +
            0.07842 tch_p_g_Right   tch_p_l_Right^4 +
                              tch_p_l_Right^5
         )^0.2;
      local #m_unique tch_q_Right = tch_p_l_Right / tch_p_Right;
      peak_type spv
         spv_l1 = 1.36603 tch_q_Left - 0.47719 tch_q_Left^2 + 0.1116 tch_q_Left^3;
         spv_h1 = tch_p_Left;
         spv_l2 = 1.36603 tch_q_Right - 0.47719 tch_q_Right^2 + 0.1116 tch_q_Right^3;
         spv_h2 = tch_p_Right;
   }     
'------------------------
  
'------------------------
macro Triplet_Split_Peak_2D( sL, sL_v, oL, oL_v,    sM, sM_v, oM, oM_v,    sR, sR_v, oR, oR_v, FDcorr, FDcorr_v)
{  	
' The "Triplet_Split_Peak_2D" macro separates the right, middle and left componenents of each Bragg reflection.
' The left and right components are described by an indipendently refinable sample displacement parameter and by the TCHZ split peak type function. 
' The middle contribution is described by a Gaussian function convolution.
' A refinable scale factor is also added to account for the different intensity of each peak component.
' A refinable correction for the use of large flat area detectors is considered.
' Authors: Paolo P. Mazzeo, University of Parma - paolopio.mazzeo[at]unipr.it; Giulio I. Lampronti, University of Cambridge - gil21[at]cam.ac.uk
' Please cite Nat Commun 12, 6134 (2021). https://doi.org/10.1038/s41467-021-26264-1

	'Initialize variables 
		#m_argu sL 'scale of left peak
		#m_argu oL 'offset of left peak
		#m_argu sM 'scale of middle peak
		#m_argu oM 'offset of middle peak
		#m_argu sR 'scale of right peak
		#m_argu oR 'offset of right peak
		#m_argu FDcorr 'correction for flat detector dimension

	   If_Prm_Eqn_Rpt(sL, sL_v, min 0, )
	   If_Prm_Eqn_Rpt(oL, oL_v, min -10 max 10,)
	   If_Prm_Eqn_Rpt(sM, sM_v,min 0, )
	   If_Prm_Eqn_Rpt(oM, oM_v,min -10 max 10,)
	   If_Prm_Eqn_Rpt(sR, sR_v,min 0, )
	   If_Prm_Eqn_Rpt(oR, oR_v,min -10 max 10,)
	   If_Prm_Eqn_Rpt(FDcorr, FDcorr_v, min -10 max 10,)
	 	
'left peak	 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_L; ,, 0.01 , @  ,=TCHZ03_L; ,, 0, @ , =TCHZ04_L; ,, 0,  @ ,=TCHZ05_L; ,  @ , =TCHZ06_L; , @ ,=TCHZ07_L; ,, 0 , @ , =TCHZ08_L; ,, 0) 	 
	Peak_Split_2Th_displacement_2D( @  ,= CeV(oL, oL_v);, @ ,= CeV(FDcorr, FDcorr_v);)  	 
	scale  = CeV(sL, sL_v)/1e10;
'middle peak	 
      str   
	User_Defined_Dependence_Convolution(gauss_fwhm,   (2 Th)   , @ , =GaussFWHM_M; min 0.1 max 1)  
	Peak_Split_2Th_displacement( @  ,= CeV(oM, oM_v); )
	scale  = CeV(sM, sM_v)/1e10;  
'right peak 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_R; ,, 0.01 ,  @ ,=TCHZ03_R; ,, 0, @ , =TCHZ04_R; ,, 0,  @ ,=TCHZ05_R; ,  @ , =TCHZ06_R; , @  ,=TCHZ07_R; ,, 0 ,@  , =TCHZ08_R; ,, 0)  
	Peak_Split_2Th_displacement_2D( @  ,= CeV(oR, oR_v);, @ ,= CeV(FDcorr, FDcorr_v);)  	 
	scale  = CeV(sR, sR_v)/1e10;
}
'------------------------

'------------------------
macro Triplet_Split_Peak( sL, sL_v, oL, oL_v,    sM, sM_v, oM, oM_v,    sR, sR_v, oR, oR_v)
{  	
' The "Triplet_Split_Peak_2D" macro separates the right, middle and left componenents of each Bragg reflection.
' The left and right components are described by an indipendently refinable sample displacement parameter and by the TCHZ split peak type function. 
' The middle contribution is described by a Gaussian function convolution.
' A refinable scale factor is also added to account for the different intensity of each peak component.
' This macro is particularly indicated for data collected in scanning mode. 
' Authors: Paolo P. Mazzeo, University of Parma - paolopio.mazzeo[at]unipr.it; Giulio I. Lampronti, University of Cambridge - gil21[at]cam.ac.uk
' Please cite Nat Commun 12, 6134 (2021). https://doi.org/10.1038/s41467-021-26264-1

	'Initialize variables 
		#m_argu sL 'scale of left peak
		#m_argu oL 'offset of left peak
		#m_argu sM 'scale of middle peak
		#m_argu oM 'offset of middle peak
		#m_argu sR 'scale of right peak
		#m_argu oR 'offset of right peak

	   If_Prm_Eqn_Rpt(sL, sL_v, min 0, )
	   If_Prm_Eqn_Rpt(oL, oL_v, min -10 max 10,)
	   If_Prm_Eqn_Rpt(sM, sM_v,min 0, )
	   If_Prm_Eqn_Rpt(oM, oM_v,min -10 max 10,)
	   If_Prm_Eqn_Rpt(sR, sR_v,min 0, )
	   If_Prm_Eqn_Rpt(oR, oR_v,min -10 max 10,)
		
'left peak	 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_L; ,, 0.01 , @  ,=TCHZ03_L; ,, 0, @ , =TCHZ04_L; ,, 0,  @ ,=TCHZ05_L; ,  @ , =TCHZ06_L; , @ ,=TCHZ07_L; ,, 0 , @ , =TCHZ08_L; ,, 0) 	 
	Peak_Split_2Th_displacement( @  ,= CeV(oL, oL_v); ) 	 
	scale  = CeV(sL, sL_v)/1e10;
'middle peak	 
      str   
	User_Defined_Dependence_Convolution(gauss_fwhm,   (2 Th)   , @ , =GaussFWHM_M; min 0.1 max 1)  
	Peak_Split_2Th_displacement( @  ,  = CeV(oM, oM_v);)  
	scale  = CeV(sM, sM_v)/1e10;  
'right peak 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_R; ,, 0.01 ,  @ ,=TCHZ03_R; ,, 0, @ , =TCHZ04_R; ,, 0,  @ ,=TCHZ05_R; ,  @ , =TCHZ06_R; , @  ,=TCHZ07_R; ,, 0 ,@  , =TCHZ08_R; ,, 0)  
	Peak_Split_2Th_displacement( @ ,= CeV(oR, oR_v);)  
	scale  = CeV(sR, sR_v)/1e10;
}
'------------------------

'------------------------
macro Doublet_Split_Peak_2D( sL, sL_v, oL, oL_v, sR, sR_v, oR, oR_v, FDcorr, FDcorr_v)
{  	
' The "Doublet_Split_Peak_2D" macro separates the right and left componenents of each Bragg reflection.
' The left and right components are described by an indipendently refinable sample displacement parameter and by the TCHZ split peak type function. 
' A refinable scale factor is also added to account for the different intensity of each peak component.
' A refinable correction for the use of large flat area detectors is considered.
' Authors: Paolo P. Mazzeo, University of Parma - paolopio.mazzeo[at]unipr.it; Giulio I. Lampronti, University of Cambridge - gil21[at]cam.ac.uk
' Please cite Nat Commun 12, 6134 (2021). https://doi.org/10.1038/s41467-021-26264-1

	'Initialize variables 
		#m_argu sL 'scale of left peak
		#m_argu oL 'offset of left peak
		#m_argu sR 'scale of right peak
		#m_argu oR 'offset of right peak

	   If_Prm_Eqn_Rpt(sL, sL_v, min 0, )
	   If_Prm_Eqn_Rpt(oL, oL_v, min -10 max 10,)
	   If_Prm_Eqn_Rpt(sR, sR_v,min 0, )
	   If_Prm_Eqn_Rpt(oR, oR_v,min -10 max 10,)
	   If_Prm_Eqn_Rpt(FDcorr, FDcorr_v, min -10 max 10,)
	 	
'left peak	 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_L; ,, 0.01 , @  ,=TCHZ03_L; ,, 0, @ , =TCHZ04_L; ,, 0,  @ ,=TCHZ05_L; ,  @ , =TCHZ06_L; , @ ,=TCHZ07_L; ,, 0 , @ , =TCHZ08_L; ,, 0) 	 
	Peak_Split_2Th_displacement_2D( @  ,= CeV(oL, oL_v);, @ ,= CeV(FDcorr, FDcorr_v);)  	 
	scale  = CeV(sL, sL_v)/1e10;
'right peak 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_R; ,, 0.01 ,  @ ,=TCHZ03_R; ,, 0, @ , =TCHZ04_R; ,, 0,  @ ,=TCHZ05_R; ,  @ , =TCHZ06_R; , @  ,=TCHZ07_R; ,, 0 ,@  , =TCHZ08_R; ,, 0)  
	Peak_Split_2Th_displacement_2D( @  ,= CeV(oR, oR_v);, @ ,= CeV(FDcorr, FDcorr_v);)  	 
	scale  = CeV(sR, sR_v)/1e10;
}
'------------------------

'------------------------
macro Doublet_Split_Peak( sL, sL_v, oL, oL_v, sR, sR_v, oR, oR_v)
{
' The "Doublet_Split_Peak" macro separates the right and left componenents of each Bragg reflection.
' The left and right components are described by an indipendently refinable sample displacement parameter and by the TCHZ split peak type function. 
' A refinable scale factor is also added to account for the different intensity of each peak component.
' This macro is particularly indicated for data collected in scanning mode. 
' Authors: Paolo P. Mazzeo, University of Parma - paolopio.mazzeo[at]unipr.it; Giulio I. Lampronti, University of Cambridge - gil21[at]cam.ac.uk
' Please cite Nat Commun 12, 6134 (2021). https://doi.org/10.1038/s41467-021-26264-1

	'Initialize variables 
		#m_argu sL 'scale of left peak
		#m_argu oL 'offset of left peak
		#m_argu sR 'scale of right peak
		#m_argu oR 'offset of right peak

	   If_Prm_Eqn_Rpt(sL, sL_v, min 0, )
	   If_Prm_Eqn_Rpt(oL, oL_v, min -10 max 10,)
	   If_Prm_Eqn_Rpt(sR, sR_v,min 0, )
	   If_Prm_Eqn_Rpt(oR, oR_v,min -10 max 10,)
		
'left peak	 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_L; ,, 0.01 , @  ,=TCHZ03_L; ,, 0, @ , =TCHZ04_L; ,, 0,  @ ,=TCHZ05_L; ,  @ , =TCHZ06_L; , @ ,=TCHZ07_L; ,, 0 , @ , =TCHZ08_L; ,, 0) 	 
	Peak_Split_2Th_displacement( @  ,= CeV(oL, oL_v); ) 	 
	scale  = CeV(sL, sL_v)/1e10;
'right peak 
      str   
	TCHZ_Split_Peak_Type( @ ,=TCHZ01_R; ,, 0.01 ,  @ ,=TCHZ03_R; ,, 0, @ , =TCHZ04_R; ,, 0,  @ ,=TCHZ05_R; ,  @ , =TCHZ06_R; , @  ,=TCHZ07_R; ,, 0 ,@  , =TCHZ08_R; ,, 0)  
	Peak_Split_2Th_displacement( @ ,= CeV(oR, oR_v);)  
	scale  = CeV(sR, sR_v)/1e10;
}
'------------------------